//
//  test.cpp
//  Computational Geometry - 2018
//
//  Requires C++11 or later.
//
//  Created by Maximilian Katzmann on 09.10.18.
//

#include "io_helper.hpp"
#include <iostream>
#include <vector>

/**
 * Main procedure
 */
int main(int argc, char *argv[]) {

  /**
   * Check whether an input file was specified.
   */
  if (argc < 2) {
    std::cerr << "No input file specified. Usage: " << argv[0]
              << " path/to/input.txt" << std::endl;
    return EXIT_FAILURE;
  }

  /**
   * The vector holding the points from the input.
   */
  std::vector<Point> points;

  /**
   * Reading the points from the file.
   */
  std::string file_name(argv[1]);
  IOHelper::read_points_from_file(file_name, points);

  /**
   * We interpret the first half of the points as a polygon and the
   * second half as single points.
   */
  std::vector<Point> polygon;
  std::vector<Point> single_points;
  for (int index = 0; index < (int)points.size(); ++index) {
    if (index < (int)points.size() / 2) {
      polygon.push_back(points[index]);
    } else {
      single_points.push_back(points[index]);
    }
  }

  /**
   * Additionally we add a line between the last two single points.
   */
  std::vector<Point> line = {single_points.back(),
                             single_points[single_points.size() - 2]};

  /**
   * Get the SVG representation of the polygon and the points.
   */
  std::string svg_representation;
  std::vector<std::vector<Point>> polygons = {polygon, line};
  IOHelper::svg_representation(single_points, polygons, svg_representation);

  /**
   * Write the SVG representation to the console.
   */
  std::cout << svg_representation;

  return EXIT_SUCCESS;
}
