#include <iostream>
#include <vector>

#include "../../code/cpp/io_helper.hpp"

int main(int argc, char *argv[]) {

  if (argc < 2) {
    std::cout << "Missing input file! Usage: " << argv[0]
              << " path/to/geometric_graph.txt" << std::endl;

    return EXIT_FAILURE;
  }

  std::string file_name(argv[1]);

  /**
   * Open the file.
   */
  std::ifstream file(file_name.c_str());
  std::string line;

  // Read the first line, which should consist of the number of nodes
  // and edges in the graph.

  int n = 0; // Nodes
  int m = 0; // Edges

  if (!std::getline(file, line)) {
    std::cerr << "Could not read first line from file." << std::endl;
    return EXIT_FAILURE;
  }

  std::vector<double> components;
  IOHelper::components_in_line(line, components);

  if (components.size() != 2) {
    std::cerr << "First line: '" << line
              << "' did not consist of 2 number specifying nodes and edges."
              << std::endl;
    return EXIT_FAILURE;
  }

  n = (int)components[0];
  m = (int)components[1];

  // At this points, we know the number of nodes and the number of
  // edges. -> Initialize the graph.
  std::vector<std::vector<int>> graph(n, std::vector<int>());

  // Initialize the coordinates;
  std::vector<Point> coordinates(n, Point());

  // Read the coordinates of the nodes.
  int read_coordinates = 0;
  while (read_coordinates < n) {

    if (!std::getline(file, line)) {
      std::cerr << "Could not read line describing node " << read_coordinates
                << " of " << n << " specified nodes." << std::endl;
      return EXIT_FAILURE;
    }

    std::vector<double> numbers_in_line;
    IOHelper::components_in_line(line, numbers_in_line);

    if (numbers_in_line.size() != 3) {
      std::cerr << "Line '" << line << "' describing the coordinates of vertex "
                << read_coordinates << " did not consist of 3 numbers."
                << std::endl;
      return EXIT_FAILURE;
    }

    // Generate node id and coordinate from the numbers in the line.
    int node_id = (int)numbers_in_line[0];
    Point coordinate = Point(numbers_in_line[1], numbers_in_line[2]);

    // Assign coordinate to node.
    coordinates[node_id] = coordinate;

    ++read_coordinates;
  }

  // The next lines should describe edges.

  int read_edges = 0;

  while (read_edges < m) {
    if (!std::getline(file, line)) {
      std::cerr << "Could not read line describing edge " << read_edges
                << " of " << m << " specified nodes." << std::endl;
      return EXIT_FAILURE;
    }

    std::vector<double> numbers_in_line;
    IOHelper::components_in_line(line, numbers_in_line);

    if (numbers_in_line.size() != 2) {
      std::cerr << "Line '" << line
                << "' describing the an edge between vertices "
                << read_coordinates << " did not consist of 2 numbers."
                << std::endl;
      return EXIT_FAILURE;
    }

    graph[(int)numbers_in_line[0]].push_back((int)numbers_in_line[1]);

    ++read_edges;
  }

  // We're done reading the graph. Now draw it. -> For that create a
  // "polygon" for every edge.
  std::vector<std::vector<Point>> edges;

  for (int i = 0; i < (int)graph.size(); ++i) {
    for (int j : graph[i]) {
      std::vector<Point> edge = {coordinates[i], coordinates[j]};
      edges.push_back(edge);
    }
  }

  std::string svg_representation;
  IOHelper::svg_representation(std::vector<Point>(), edges, svg_representation);

  std::cout << svg_representation << std::endl;

  return EXIT_SUCCESS;
}
